/*
    Copyright (C) 2004, 2005  Vansteenhuyse Hubert <hvansteenhuyse@yahoo.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "subnet.h"
char out[81];
int arg_stdout = 0, arg_usenet = 0, arg_usehost = 0, sw_pub = 1;
unsigned long ip = 0, mask = 0, wild = 0;

char *publicity = "";

char *help[] = {
    "subnet [options] [IP] [SUBNETMASK]",
    "Options:",
    "--stdout\t\tRuns in non interactive modus",
    "--hosts [amount]\tSubnet listing for specified amount of hosts",
    "--nets [amount]\t\tSubnet listing for specified amount of networks",
    "--version\t\tShows the version",
    "--nopub\t\t\tSupresses the publicity when exiting the program\n",
    "--help\t\t\tShows this help message\n",
    "Notice: Options --hosts and --nets can't be used together.\n",
    NULL
};

void yield(void)
{
#ifdef __WINDOWS__
    MSG msg;
        if(PeekMessage(&msg, (HWND)0, 0, 0, PM_REMOVE)){
           TranslateMessage(&msg);
    	   DispatchMessage(&msg);
    	}
#endif				/*#__WINDOWS__ */
}


void writefile(FILE * stream)
{
    int sw_histogram = (stream == stdout ? 0 : 1), c = 0;
    unsigned long net, cast, count = 1, totalnets = getnets();

    fprintf(stream, "\nSubnet Calculator %d.%d\n", VERSION_HI, VERSION_LO);
    for (c = 0; c < 80; c++)
	fprintf(stream, "-");
    fprintf(stream, "\n");
    fprintf(stream, "cidr: %s / %d \n", longtoip(ip), calcbits(mask));
    fprintf(stream, "netmask: %s\n", longtoip(mask));
    fprintf(stream, "inverse netmask: %s\n", longtoip_inverse(mask));
    fprintf(stream, "class %c [%s]\n", getipclass(ip), iptype(ip));
    fprintf(stream, "#networks: %u, ", (unsigned) getnets());
    fprintf(stream, "#hosts: %u\n", (unsigned) gethosts());
    fprintf(stream, "magic number: %d, ", magicnumber());
    fprintf(stream, "#subnetbits: %d, %s\n", getsubnetbits(), getipnetbits());
    for (c = 0; c < 80; c++)
	fprintf(stream, "-");
    fprintf(stream, "\n");
    fprintf(stream,
	    "#        | NET             | BCAST           | RANGE                            \n");
    for (c = 0; c < 80; c++)fprintf(stream, "-");
    fprintf(stream, "\n");
    while (count <= totalnets) {
        if (sw_exit) return;

	net = ipcalcnet(count);
	cast = ipcalcbcast(count);
	memset(out, 0, 81);

	sprintf(out, "%u", (unsigned) count);
	while (strlen(out) < 9) sprintf(out + strlen(out), " ");
	sprintf(out + strlen(out), "| ");

	sprintf(out + strlen(out), "%s", longtoip(net));
	while (strlen(out) < 27) sprintf(out + strlen(out), " ");
	sprintf(out + strlen(out), "| ");
	sprintf(out + strlen(out), "%s", longtoip(cast));
	while (strlen(out) < 45) sprintf(out + strlen(out), " ");
	sprintf(out + strlen(out), "| ");
	sprintf(out + strlen(out), "%s - ", longtoip(net + 1));
	sprintf(out + strlen(out), "%s", longtoip(cast - 1));
	fprintf(stream, "%s\n", out);

	if (sw_histogram) histogram(count, totalnets);
	count++;
    }
}

//show help & exit 
void helpermsg(void)
{
    int h = 0;
    while (help[h] != NULL) {
	fprintf(stdout, "%s\n", help[h]);
	h++;
    }
    if (sw_pub)
	fprintf(stdout, "%s\n", publicity);
    exit(1);
}


int main(int argc, char *argv[])
{
    int arg = 1;

    ip = iptolong("10.0.0.1");
    mask = iptolong("255.0.0.0");
    wild = inverse_mask(mask);

    while (arg < argc) {
	if (strcmp(argv[arg], "--help") == 0) {
	    helpermsg();
	}
	if (strcmp(argv[arg], "--nopub") == 0) {
	    sw_pub = 0;
	    arg++;
	    continue;
	}
	if (strcmp(argv[arg], "--stdout") == 0) {
	    arg_stdout = 1;
	    arg++;
	    continue;
	}
	if (strcmp(argv[arg], "--version") == 0) {
	    fprintf(stdout, "subnet calculator version %d.%d\n",
		    VERSION_HI, VERSION_LO);
	    exit(0);
	}


	if (strcmp(argv[arg], "--nets") == 0) {
	    arg++;
	    if (arg >= argc) {
		fprintf(stderr,
			"in parameter --nets the amount of networks is missing...\n\n");
		helpermsg();
	    }
	    arg_usenet = (long) atoi(argv[arg]);
	    arg_usehost = 0;
	    arg++;
	    continue;
	}

	if (strcmp(argv[arg], "--hosts") == 0) {
	    arg++;
	    if (arg >= argc) {
		fprintf(stderr,
			"in parameter --hosts the amount of hosts is missing...\n\n");
		helpermsg();
	    }
	    arg_usehost = (long) atoi(argv[arg]);
	    arg_usenet = 0;
	    arg++;
	    continue;
	}

	if (!isvalidip(argv[arg])) {
	    fprintf(stderr, "Invalid IP address %s \n\n", argv[arg]);
	    helpermsg();
	}
	ip = iptolong(argv[arg]);
	arg++;
	if (arg >= argc) {
	    fprintf(stderr, "Subnetmask is missing...\n\n");
	    helpermsg();
	}

	if (!isvalidmask(argv[arg])) {
	    fprintf(stderr, "Invalid subnetmask %s\n\n", argv[arg]);
	    helpermsg();
	}
	mask = iptolong(argv[arg]);
	arg++;
    }

    if (arg_usenet)
	useamount_net(arg_usenet);
    if (arg_usehost)
	useamount_host(arg_usehost);

    if (!arg_stdout) {
	//Run interactively
	if (initScreen()) {
	    updateScreen();
	    input();
	    exitScreen();
	} else
	    fprintf(stderr, "Cannot initialize ncurses...");
    } else {
	//Process all the output to stdout...
	writefile(stdout);
    }

    //fprintf(stdout, "\n");
    //if(sw_pub) fprintf(stdout, "%s\n", publicity);

    return EXIT_SUCCESS;
}
