/*
    Copyright (C) 2004, 2005  Vansteenhuyse Hubert <hvansteenhuyse@yahoo.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <stdarg.h>
#include <math.h>

#ifdef __LINUX__
#include <sys/signal.h>
#include <ncurses.h>
#endif

extern int sw_exit;

#ifdef __WINDOWS__
#include <windows.h>
extern char win_kbd;
#endif


#define VERSION 0x101		//versions
#define VERSION_HI ((0x0FF00 & VERSION) >> 8)
#define VERSION_LO (0x00FF & VERSION)

//ISDIGIT onderzoekt of een karakter numeric is
#define ISDIGIT(c)( (c < '0' || c > '9') ? 0 : 1)

//PROCENT Geeft het percentage van CUR tov MAX
#define PROCENT(CUR, MAX)(MAX == 0 ? 100 : (100.0/MAX*CUR))

//PROCENT Geeft huidig cijfer voor PROCENT tov MAX
#define CUR_PROCENT(PCT, MAX)(MAX == 0 ? 0 : (MAX/100.0*PCT))

unsigned long ip, mask, wild;
/* --- main.c declarations --- */
extern void writefile(FILE * stream);
extern void yield(void);

/* --- screen.c declarations --- */
extern int initScreen(void);
extern void exitScreen(void);
extern void updateScreen(void);
extern void input(void);
extern void histogram(unsigned long curr, unsigned long max);
extern int get_x(void);
extern int get_y(void);
extern int get_maxx(void);
extern int get_maxy(void);

/* --- keyb.c declarations --- */
#define NOKEY 0
#define KEYU 1
#define KEYD 2
#define KEYR 3
#define KEYL 4
#define KEYBACK 5
#define PGD 6
#define PGU 7

extern char getinput(void);


/* --- ipcalc.c declarations --- */
long inverse_mask(long msk);
int is_valid_inverse_mask(long imask);


	//returns class ABCDE of mask
extern char getipclass(long ip);
	//returns the amount of bits used in a mask
extern int calcbits(long mask);
	//returns the ip notation (eg 127.0.0.1) from a long
extern char *longtoip(long ip);
	//returns the long for an ip eg: 127.0.0.1
extern long iptolong(char *ip);

//returns the inverse ip notation (eg 127.0.0.1) from a long
extern char *longtoip_inverse(long ip);

	//returns the netmask for a cidr mask (eg: 8 = 255.0.0.0 converted to its long value
extern long cidrtomask(int thecidrmask);

	//returns a string like nnnnnnn.nnnnnnn.hhhhhhhhh,hhhhh... based on the mask
extern char *getipnetbits(void);

//returns 10101010.10101010.10101010.10101010 for an ip
char* getiptobin(char*ip);

	//returns the digit of an ip address. eg: digit 2 of 193.19.136.1 = 19
extern int getipdigit(long ipaddress, int digit);
	//returns the magic number (= 256 minus LAST NONZERO NETMASK BYTE)
extern int magicnumber(void);
	//returns the private/public, apipa or loopback type
extern char *iptype(int ipaddress);
	//returns the amount of subnet bits used
extern int getsubnetbits(void);
extern long getnets(void);
extern long gethosts(void);

extern int isvalidip(char *ip);
extern int isvalidmask(char *ip);

	//calculates the network address of the specified network
extern long ipcalcnet(long network);
	//calculates the broadcast address of the specified network
extern long ipcalcbcast(long network);

extern void useamount_net(unsigned long nets);
extern void useamount_host(unsigned long hosts);
extern unsigned long findnet(long theip);
